"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Generated by CoffeeScript 2.7.0
(function () {
  var AWS, MESSAGE_PROPERTIES, MonitoringCenterObserver, Promise, TYPE_PROPERTIES, _, debug, moment, retry;

  _ = require("lodash");

  Promise = require("bluebird");

  retry = require("bluebird-retry");

  debug = require("debug")("notification-processor:observers:monitor-center");

  AWS = require("aws-sdk");

  moment = require("moment");

  TYPE_PROPERTIES = ["cause.type", "type"];

  MESSAGE_PROPERTIES = ["cause.message", "message"];

  module.exports = MonitoringCenterObserver = function () {
    function MonitoringCenterObserver(_ref) {
      var sender = _ref.sender,
          clientId = _ref.clientId,
          app1 = _ref.app,
          job1 = _ref.job,
          _ref$propertiesToOmit = _ref.propertiesToOmit,
          propertiesToOmit = _ref$propertiesToOmit === undefined ? "auth" : _ref$propertiesToOmit,
          _ref$connection = _ref.connection,
          accessKeyId = _ref$connection.accessKeyId,
          secretAccessKey = _ref$connection.secretAccessKey,
          deliveryStream = _ref$connection.deliveryStream,
          region = _ref$connection.region;

      _classCallCheck(this, MonitoringCenterObserver);

      this.registerRecord = this.registerRecord.bind(this);
      this.sender = sender;
      this.clientId = clientId;
      this.app = app1;
      this.job = job1;
      this.propertiesToOmit = propertiesToOmit;
      this.deliveryStream = deliveryStream;
      this.firehose = new AWS.Firehose({ accessKeyId: accessKeyId, secretAccessKey: secretAccessKey, region: region });
      this.uploadToFirehose = Promise.promisify(this.firehose.putRecord).bind(this.firehose);
    }

    _createClass(MonitoringCenterObserver, [{
      key: "listenTo",
      value: function listenTo(observable) {
        var _this = this;

        observable.on("unsuccessful_non_retryable", function (payload) {
          return _this.registerRecord(payload, "unsuccessful");
        });
        observable.on("unsuccessful", function (payload) {
          return _this.registerRecord(payload, "unsuccessful");
        });
        observable.on("started", function (payload) {
          return _this.registerRecord(payload, "pending");
        });
        return observable.on("successful", function (payload) {
          return _this.registerRecord(payload, "successful");
        });
      }
    }, {
      key: "registerRecord",
      value: function registerRecord(payload, executionStatus) {
        var _this2 = this;

        return this._mapper(_.merge({ executionStatus: executionStatus }, payload)).tap(function (record) {
          return debug("Record to save in firehose %s %j", _this2.deliveryStream, record);
        }).then(function (record) {
          var __uploadToFirehose, uploadParams;
          if (_.isEmpty(record)) {
            return;
          }
          uploadParams = {
            DeliveryStreamName: _this2.deliveryStream,
            Record: {
              Data: JSON.stringify(record)
            }
          };
          debug("Uploading record " + record.event + "/" + record.id + " to firehose delivery stream " + uploadParams.DeliveryStreamName);
          __uploadToFirehose = function __uploadToFirehose() {
            return _this2.uploadToFirehose(uploadParams);
          };
          return retry(__uploadToFirehose, {
            throw_original: true
          }).tap(function () {
            return debug("Uploaded record " + record.event + "/" + record.id + " to firehose delivery stream " + uploadParams.DeliveryStreamName);
          }).catch(function (e) {
            // We'll do nothing with this error
            return debug("Error uploading record " + record.event + "/" + record.id + " to firehose delivery stream " + uploadParams.DeliveryStreamName + " %o", e);
          });
        });
      }
    }, {
      key: "_mapper",
      value: function _mapper(_ref2) {
        var _this3 = this;

        var id = _ref2.id,
            notification = _ref2.notification,
            error = _ref2.error,
            warnings = _ref2.warnings,
            executionStatus = _ref2.executionStatus;

        return Promise.method(this.sender.monitoringCenterFields.bind(this.sender))(notification).then(function (_ref3) {
          var eventType = _ref3.eventType,
              resource = _ref3.resource,
              companyId = _ref3.companyId,
              userId = _ref3.userId,
              externalReference = _ref3.externalReference,
              userExternalReference = _ref3.userExternalReference,
              eventId = _ref3.eventId,
              eventTimestamp = _ref3.eventTimestamp,
              parentEventId = _ref3.parentEventId,
              app = _ref3.app,
              job = _ref3.job,
              partialMessage = _ref3.partialMessage;

          var errorMessage, errorType, now, theRequest;
          if (!eventId) {
            return Promise.resolve({});
          }
          theRequest = _.get(error, "detail.request") || _.get(error, "cause.detail.request");
          errorType = _this3._retrieveMessageFromError(error, TYPE_PROPERTIES, "unknown");
          errorMessage = _this3._retrieveMessageFromError(error, MESSAGE_PROPERTIES, "");
          now = new Date();
          return {
            id: id,
            executionId: id,
            app: app || parseInt(_this3.clientId) || null,
            type: eventType || "service-bus",
            company: companyId,
            user: userId,
            event: eventId,
            parent: parentEventId,
            externalreference: externalReference,
            userexternalreference: userExternalReference,
            timestamp: now.getTime(),
            date: now.toISOString(),
            year: moment(now).format('YYYY'),
            month: moment(now).format('MM'),
            day: moment(now).format('DD'),
            hour: moment(now).format('HH'),
            payload: JSON.stringify({
              clientId: _this3.clientId,
              job: _this3.job,
              app: _this3.app,
              error: _.omit(error, ["detail.request", "cause.detail.request"]),
              request: _.omit(theRequest, _.castArray(_this3.propertiesToOmit).concat("auth")),
              tags: _.get(error, "tags", []),
              message: partialMessage || notification.message
            }),
            status: executionStatus,
            resource: resource,
            integration: _this3.app + "|" + (job || _this3.job),
            // Generic app fields
            event_timestamp: eventTimestamp || now.getTime(),
            error_type: errorType,
            output_message: errorMessage,
            user_settings_version: null, //TODO
            env_version: null, //TODO
            code_version: null, //TODO
            warnings: warnings != null ? warnings.map(function (warning) {
              return {
                type: _this3._retrieveMessageFromError(warning, TYPE_PROPERTIES, "unknown"),
                message: _this3._retrieveMessageFromError(warning, MESSAGE_PROPERTIES, "")
              };
            }) : void 0
          };
        });
      }
    }, {
      key: "_retrieveMessageFromError",
      value: function _retrieveMessageFromError(error, properties, defaultValue) {
        return error && _(properties).map(function (property) {
          return _.get(error, property);
        }).reject(_.isEmpty).get(0, defaultValue);
      }
    }]);

    return MonitoringCenterObserver;
  }();
}).call(undefined);