"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// Generated by CoffeeScript 2.7.0
(function () {
  var MeliSender, Processor, ProcessorBuilder, ProductecaSender, QueueSource, ServiceBusSource, UnknownSource, _, logger;

  _ = require("lodash");

  Processor = require("./processor");

  logger = require("./observers/logger.observer");

  var _require = require("./sources");

  UnknownSource = _require.UnknownSource;
  ServiceBusSource = _require.ServiceBusSource;
  QueueSource = _require.QueueSource;

  var _require2 = require("./senders");

  MeliSender = _require2.MeliSender;
  ProductecaSender = _require2.ProductecaSender;


  ProcessorBuilder = function () {
    function ProcessorBuilder() {
      _classCallCheck(this, ProcessorBuilder);

      this.source = UnknownSource;
      this.listeners = [];
    }

    _createClass(ProcessorBuilder, [{
      key: "withSource",
      value: function withSource(source) {
        this.source = source;
        return this;
      }
    }, {
      key: "withSender",
      value: function withSender(sender) {
        this.sender = sender;
        return this;
      }
    }, {
      key: "withTimeout",
      value: function withTimeout(timeout) {
        this.timeout = timeout;
        return this;
      }
    }, {
      key: "withApm",
      value: function withApm(apm) {
        this.apm = apm;
        return this;
      }
    }, {
      key: "withDelayObserver",
      value: function withDelayObserver(opts) {
        return this.withListeners(this.source.delayObserver(opts));
      }
    }, {
      key: "withDeadLetterSucceeded",
      value: function withDeadLetterSucceeded(opts) {
        if (this.sender == null) {
          throw new Error("Sender is required");
        }
        return this.withListeners(this.source.deadLetterSucceeded(_.defaults(opts, { sender: this.sender })));
      }
    }, {
      key: "withDidLastRetry",
      value: function withDidLastRetry(opts) {
        if (this.sender == null) {
          throw new Error("Sender is required");
        }
        return this.withListeners(this.source.didLastRetry(_.defaults(opts, { sender: this.sender })));
      }
    }, {
      key: "fromServiceBus",
      value: function fromServiceBus() {
        return this.withSource(ServiceBusSource);
      }
    }, {
      key: "fromQueue",
      value: function fromQueue() {
        return this.withSource(QueueSource);
      }
    }, {
      key: "fromMeli",
      value: function fromMeli() {
        return this.withSender(MeliSender);
      }
    }, {
      key: "fromProducteca",
      value: function fromProducteca() {
        return this.withSender(ProductecaSender);
      }
    }, {
      key: "withFunction",
      value: function withFunction(command) {
        this.command = command;
        return this;
      }
    }, {
      key: "withLogging",
      value: function withLogging() {
        return this.withListeners(logger);
      }
    }, {
      key: "withListeners",
      value: function withListeners() {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        this.listeners = _.concat(this.listeners, args);
        return this;
      }
    }, {
      key: "build",
      value: function build() {
        var processor;
        processor = new Processor({
          source: this.source,
          runner: this.command,
          timeout: this.timeout,
          apm: this.apm
        });
        _.forEach(this.listeners, function (listener) {
          return listener.listenTo(processor);
        });
        return processor;
      }
    }], [{
      key: "create",
      value: function create() {
        return new this();
      }
    }]);

    return ProcessorBuilder;
  }();

  module.exports = ProcessorBuilder;
}).call(undefined);