"use strict";

// Generated by CoffeeScript 2.7.0
(function () {
  var IgnoredError, MESSAGE_PROPERTIES, NonRetryableError, Promise, RequestError, StatusCodeError, _, __isIncludedInStatusesError, _safeParse, _type, errorConditions, httpStatus, request;

  _ = require("lodash");

  NonRetryableError = require("../exceptions/non.retryable");

  IgnoredError = require("../exceptions/ignored.error");

  Promise = require("bluebird");

  request = require("request-promise");

  var _require = require("request-promise/errors");

  StatusCodeError = _require.StatusCodeError;
  RequestError = _require.RequestError;


  httpStatus = require("http").STATUS_CODES;

  MESSAGE_PROPERTIES = ["reason", "error.error", "error.code", "code", "error"];

  _safeParse = function _safeParse(raw) {
    if (_.isObject(raw)) {
      return raw;
    } else {
      try {
        return JSON.parse(raw);
      } catch (error1) {}
    }
  };

  _type = function _type(statusCode, error) {
    return _(MESSAGE_PROPERTIES).map(function (key) {
      return _.get(error, key);
    }).concat([_.toLower(httpStatus[statusCode])]).filter(_.isString).compact().head();
  };

  errorConditions = {
    client: function client(it) {
      return it >= 400 && it < 500;
    },
    server: function server(it) {
      return it >= 500;
    }
  };

  __isIncludedInStatusesError = function __isIncludedInStatusesError(conditions) {
    return function (err) {
      var statusCode;
      statusCode = _.get(err, "detail.response.statusCode");
      return _(conditions).map(function (it) {
        return _.get(errorConditions, it, _.partial(_.isEqual, it));
      }).some(function (condition) {
        return condition(statusCode);
      });
    };
  };

  module.exports = function (requestGenerator) {
    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref$silentErrors = _ref.silentErrors,
        silentErrors = _ref$silentErrors === undefined ? [] : _ref$silentErrors,
        _ref$nonRetryable = _ref.nonRetryable,
        nonRetryable = _ref$nonRetryable === undefined ? [] : _ref$nonRetryable;

    return function (notification, context, executionId) {
      return Promise.method(requestGenerator)(notification, context, executionId).then(function (options) {
        return request(options).promise().catch(RequestError, function (_ref2) {
          var cause = _ref2.cause;

          throw {
            type: cause.code,
            detail: cause
          };
        }).catch(StatusCodeError, function (_ref3) {
          var statusCode = _ref3.statusCode,
              error = _ref3.error;

          var safeError, type;
          safeError = _safeParse(error);
          type = _type(statusCode, safeError);
          throw {
            type: type,
            message: _.get(safeError, "error.message") || _.get(safeError, "message") || type,
            detail: {
              response: {
                statusCode: statusCode,
                body: safeError
              }
            },
            tags: safeError != null ? safeError.tags : void 0
          };
        }).tapCatch(function (err) {
          return _.defaultsDeep(err, {
            type: "unknown",
            message: "unknown",
            detail: {
              request: options
            }
          });
        }).catch(__isIncludedInStatusesError(silentErrors), function (err) {
          throw new IgnoredError("An error has ocurred in that request but should be ignored", _.omit(err, "response"));
        }).catch(__isIncludedInStatusesError(nonRetryable), function (err) {
          throw new NonRetryableError("An error has ocurred in that request", _.omit(err, "response"));
        });
      });
    };
  };
}).call(undefined);